/*******************************************************
*        Trabalho de Inteligncia Artificial           *
*             Anderson de Rezende Rocha                *
*                 Jlio Csar Alves                    * 
*******************************************************/

#ifndef ROBO_H
#define ROBO_H

#include "mina.h"
#include "deposito.h"
#include <list>
#include "ponto.h"
#include <time.h>

template<class TAmbiente>
class clRobo
{
    public:
        // Construtor do robo
        clRobo();
        // Representacao do ambiente
        clRobo(TAmbiente *umAmbiente);
        clRobo(TAmbiente *umAmbiente, int idRobo, clPonto pos, int capacidadeDeCarga = 1, bool estaCarregado = false);
        // Construtor de cpia    
        clRobo(const clRobo &robo);
        // Destrutor
        ~clRobo() {}
        // Rotina que faz o robo andar
        bool Andar();
        // Verifica se um robo est ou no carregado
        bool Carregado();
        // Retorna a capacidade de um robo
        int GetCapacidade();
        // Id do robo
        int GetId();
        // Posicao atual do robo
        clPonto GetPosicao();
        // Configura as variaveis de ambiente para funcionamento do algoritmo tabu_search
      void SetConfTabu(int raio_tabu, int nro_pontos_aleatorios, int raio_procura);
        // Verifica se um robo  igual a outro
        bool operator==(clRobo<TAmbiente> robo) { return (id == robo.id); }

    private:
        // Construtor auxiliar de um robo
        void Criar(TAmbiente *umAmbiente, int idRobo, clPonto pos, int capacidadeDeCarga = 1, bool estaCarregado = false);
        // Permite um robo carregar-se
        bool Carregar();
        // Esvazia o carregamento de um robo
        bool Esvaziar();
        // Pra um robo
        void Parar();
        // Encontra a mina mais proxima da pos atual
        int EncontrarMinaMaisProxima();
        // Encontra o deposito mais proximo da pos atual
        int EncontrarDepositoMaisProximo();
        // Atualiza a posicao de um dado robo no ambiente
        void AtualizarPosicao(clPonto pos);
        // Escolhe um ponto para o algoritmo tabu_search
        clPonto EscolherPonto();
        // Verifica se um ponto est ou no na lista tabu
        bool PontoEhTabu(clPonto ponto);
        // Limpa a lista tabu
        void EsvaziaListaTabu();

        // identificador nico deste robo
        int id;
        // Posio do robo no ambiente
        clPonto posicao;
        // Capacidade de carregamento deste robo
        int capacidade;
        // O robo esta carregado ou vazio
        bool carregado;
        // Referncia ao ambiente em que o robo est
        TAmbiente *ambiente;
        // Ponto ao qual o robo est se dirigindo ou deve se dirigir
        clPonto pontoAlvo;
      
        // Mina da qual o robo esta retirando minerais atualmente
        int idMinaAtual;
        // Deposito ao qual o robo esta vinculado atualmente
        int idDepositoAtual;

        // Lista dos pontos tabu para a configurao atual do robo
        list<clPonto> listaTabu;
        // O quadrado de lado = raioTabu com origem em um ponto tabu  uma regio tabu
      int nroPontosAleatorios;
        // Raio utilizado para delimitar a regio onde o robo pode lanar pontos aleatrios
      int raioProcura;
        // Raio utilizado para delimitar a regio que deve ser considerada tabu pelo robo
        int raioTabu;
};

// Construtor padro
template<class TAmbiente>
clRobo<TAmbiente>::clRobo()
{
    Criar(NULL, -1, clPonto(0,0), 0, false);
}

// Construtor a partir de um ambiente j construdo
template<class TAmbiente>
clRobo<TAmbiente>::clRobo(TAmbiente *umAmbiente)
{
    Criar(umAmbiente, -1, clPonto(0,0), 0, false);
}

// Construtor a partir de um ambiente construdo e caractersticas iniciais deste robo
template<class TAmbiente>
clRobo<TAmbiente>::clRobo(TAmbiente *umAmbiente, int idRobo, clPonto pos, int capacidadeDeCarga, bool estaCarregado)
{
    Criar(umAmbiente, idRobo, pos, capacidadeDeCarga, estaCarregado);
}

// Rotina auxiliar para criar o robo efetivamente
template<class TAmbiente>
void clRobo<TAmbiente>::Criar(TAmbiente *umAmbiente, int idRobo, clPonto pos, int capacidadeDeCarga, bool estaCarregado)
{
    id = idRobo;
    posicao = pos;
    capacidade = capacidadeDeCarga;
    carregado = estaCarregado;
    idMinaAtual = -1;
    idDepositoAtual = -1;
    ambiente = umAmbiente;
    raioTabu = 1;
   nroPontosAleatorios = 5;
   raioTabu = 5;
    pontoAlvo.Set(-1, -1);
}

// Construtor de cpia
template<class TAmbiente>
clRobo<TAmbiente>::clRobo(const clRobo &robo)
{
    id = robo.id;
    posicao = robo.posicao;
    capacidade = robo.capacidade;
    carregado = robo.carregado;
    idMinaAtual = robo.idMinaAtual;
    idDepositoAtual = robo.idDepositoAtual;
    ambiente = robo.ambiente;
    raioTabu = robo.raioTabu;
   nroPontosAleatorios = robo.nroPontosAleatorios;
   raioProcura = robo.raioProcura;
    pontoAlvo = robo.pontoAlvo;
}

// Seta as configuraes do algoritmo tabu_search
template<class TAmbiente>
void clRobo<TAmbiente>::SetConfTabu(int raio_tabu, int nro_pontos_aleatorios, int raio_procura)
{
   raioTabu = raio_tabu;
   nroPontosAleatorios = nro_pontos_aleatorios;
   raioProcura = raio_procura;
}

// Faz o robo andar efetivamente
template<class TAmbiente>
bool clRobo<TAmbiente>::Andar()
{
    bool andou = true;

    if(pontoAlvo.GetX() != -1)
    {
        if(GetPosicao() == pontoAlvo || (carregado && ambiente->Deposito(idDepositoAtual)->Cheio())
            || (!carregado && ambiente->Mina(idMinaAtual)->Vazia()))
            pontoAlvo.Set(-1, -1);
        else
            AtualizarPosicao(pontoAlvo);
    }
    else if(!carregado)
    {
        if(idMinaAtual == -1 || ambiente->Mina(idMinaAtual)->Vazia())
            idMinaAtual = EncontrarMinaMaisProxima();

        if(idMinaAtual != -1)
        {
            if(GetPosicao().PertenceARegiao(ambiente->Mina(idMinaAtual)->GetPosicao(), 31))
            {
                EsvaziaListaTabu();
                Carregar();
            }
            else
                AtualizarPosicao(ambiente->Mina(idMinaAtual)->GetPosicao());
        }
        else
            andou = false;
    }
    else
    {
        if (idDepositoAtual == -1 || ambiente->Deposito(idDepositoAtual)->Cheio())
            idDepositoAtual = EncontrarDepositoMaisProximo();

        if(idDepositoAtual != -1)
        {
            if(GetPosicao().PertenceARegiao(ambiente->Deposito(idDepositoAtual)->GetPosicao(), 31))
            {
                EsvaziaListaTabu();
                Esvaziar();
            }
            else
                AtualizarPosicao(ambiente->Deposito(idDepositoAtual)->GetPosicao());
        }
        else
            andou = false;
    }
    if(!andou)
    {
        idDepositoAtual = -1;
        idMinaAtual = -1;
        pontoAlvo.Set(-1,-1);
    }
    return andou;
}

// Carrega um robo com uma quantidade de mineral
template<class TAmbiente>
bool clRobo<TAmbiente>::Carregar()
{
    if (!Carregado())
    {
        carregado = true;
        // Do jeito que a mina esta implementada se a capacidade do robo
        // for 3 e tiver 2 na mina, o robo nao eh carregado com nada
        ambiente->Mina(idMinaAtual)->RetirarOuro(GetCapacidade());
        idMinaAtual = -1;
        return true;
    }
    return false;
}

// Esvazia o robo colocando o material no deposito vinculado atualmente ao robo, caso 
// este ainda esteja apto (no cheio).
template<class TAmbiente>
bool clRobo<TAmbiente>::Esvaziar()
{
    if (Carregado())
    {
        carregado = false;
        ambiente->Deposito(idDepositoAtual)->DepositarOuro(GetCapacidade());
        idDepositoAtual = -1;
        return true;
    }
    return false;
}

// O robo esta carregado?
template<class TAmbiente>
bool clRobo<TAmbiente>::Carregado()
{
    return carregado;
}

// Qual a capacidade deste robo?
template<class TAmbiente>
int clRobo<TAmbiente>::GetCapacidade()
{
    return capacidade;
}

// Qual o identificador deste robo?
template<class TAmbiente>
int clRobo<TAmbiente>::GetId()
{
    return id;
}

// Qual a posicao atualmente ocupada por este robo?
template<class TAmbiente>
clPonto clRobo<TAmbiente>::GetPosicao()
{
    return posicao;
}

// Retorna a mina mais prxima deste robo. Utilizado para otimizao
template<class TAmbiente>
int clRobo<TAmbiente>::EncontrarMinaMaisProxima()
{
    double distancia = INFINITO;
    double novaDistancia;
    int idMina = -1;

    for (int i = 0; i < ambiente->GetNroMinas(); i++)
    {
        if (!ambiente->Mina(i)->Vazia())
        {
            novaDistancia = GetPosicao().CalculaDistancia(ambiente->Mina(i)->GetPosicao());
            if (novaDistancia < distancia)
            {
                distancia = novaDistancia;
                idMina = i;
            }
        }
    }
    return idMina;
}

// Dado que o robo esteja carregado, qual o deposito mais proximo para que 
// ele possa descarregar?
template<class TAmbiente>
int clRobo<TAmbiente>::EncontrarDepositoMaisProximo()
{
    double distancia = INFINITO;
    double novaDistancia;
    int idDeposito = -1;

    for (int i = 0; i < ambiente->GetNroDepositos(); i++)
    {
        if (!ambiente->Deposito(i)->Cheio())
        {
            novaDistancia = GetPosicao().CalculaDistancia(ambiente->Deposito(i)->GetPosicao());
            if (novaDistancia < distancia)
            {
                distancia = novaDistancia;
                idDeposito = i;
            }
        }
    }
    return idDeposito;
}

// Pode melhorar, pois o robo nao esta fazendo o melhor caminho
// Proposta para melhoria: calcular a distncia de um ponto  reta 
template<class TAmbiente>
void clRobo<TAmbiente>::AtualizarPosicao(clPonto pos)
{
    int x, y;

    x = posicao.GetX();
    y = posicao.GetY();
    ambiente->AtualizarRegiao(posicao.GetX(), posicao.GetY(), 31, 31, false);

    if (pos.GetX() > posicao.GetX())
        x++;
    else if (pos.GetX() < posicao.GetX())
        x--;

    if (pos.GetY() > posicao.GetY())
        y++;
    else if (pos.GetY() < posicao.GetY())
        y--;

    if(!ambiente->AtualizarRegiao(x, y, 31, 31, true))
    {
        ambiente->AtualizarRegiao(posicao.GetX(), posicao.GetY(), 31, 31, true);
        pontoAlvo = EscolherPonto();
    }
    else
        posicao.Set(x, y);
}

// Lana um ponto aleatrio dentro do raio de procura
template<class TAmbiente>
clPonto clRobo<TAmbiente>::EscolherPonto()
{
    int x, y, nroMaxIteracoes;
    double novaDistancia, menorDistancia;
    // PontoDestino eh o ponto de uma mina ou de um deposito
    clPonto ponto, pontoDestino, pontoCandidato;

    if(carregado)
        pontoDestino = ambiente->Deposito(idDepositoAtual)->GetPosicao();
    else
        pontoDestino = ambiente->Mina(idMinaAtual)->GetPosicao();

    // Deste modo nunca se piora a posicao. Para piora-la basta iniciar a menorDistancia como INFINITO
    menorDistancia = INFINITO;
    pontoCandidato = GetPosicao();
    nroMaxIteracoes = 100;
   int raio, j;

    // Pode-se forar que ao chegar ao numero maximo de iteracoes os novos pontos pertencem
    // a diferenca entre a nova e a antiga rea a area pode dobrar a cada certo limite de iteracoes
    for(int i = 0; i < nroPontosAleatorios; i++)
    {
      raio = raioProcura;
        // Para evitar que alguns robos tenham comportamentos padro. 
        srand(time(0)-GetId()); // +1  para evitar divisao por zero
        j = 1;
        do
        {
            // Nmero mximo de iteraes atingido. Reiniciar processo...
            if(j == nroMaxIteracoes)
            {
                raio+= raio;
                cout << "Max iteracoes" << endl;
            }

            x = rand()%(2*raio);
            y = rand()%(2*raio);
            ponto.Set(posicao.GetX()+x-raio, posicao.GetY()+y-raio);
            j++;
        } while(PontoEhTabu(ponto));
        novaDistancia = ponto.CalculaDistancia(pontoDestino);
        if(novaDistancia < menorDistancia)
        {
            pontoCandidato = ponto;
            menorDistancia = novaDistancia;
        }
        listaTabu.insert(listaTabu.end(), ponto);
    }
    listaTabu.remove(pontoCandidato);
    return pontoCandidato;
}

// Verifica se um dado ponto lanado est ou no na lista tabu
template<class TAmbiente>
bool clRobo<TAmbiente>::PontoEhTabu(clPonto ponto)
{
    list<clPonto>::iterator it;

    it = listaTabu.begin();
    while( it != listaTabu.end() && ponto.PertenceARegiao((*it), raioTabu))
        it++;
    return !(it == listaTabu.end());
}

// Esvazia os pontos atualmente na lista tabu
template<class TAmbiente>
void clRobo<TAmbiente>::EsvaziaListaTabu()
{
    list<clPonto>::iterator it;

    it = listaTabu.begin();
    while(it != listaTabu.end())
    {
        listaTabu.remove(*it);
        it = listaTabu.begin();
    }
}

#endif
