/*******************************************************
*        Trabalho de Inteligncia Artificial           *
*             Anderson de Rezende Rocha                *
*                 Jlio Csar Alves                    * 
*******************************************************/

/*******************************************************
*             INSTRUES PARA COMPILAO               *
*   Em Linux:                                          *
*       $ make -f makefile.unx                         *  
*                                                      *
*   Em Windows:                                        *
*       $ make -f makefile.win                         *
*******************************************************/

// robosMineradores.cpp
//
// Contm as classes da interface grfica e aplicao 


// Includes normais da wxWindows
// Para compiladores que suportam pre-compilao.
#include "wx/wxprec.h"
#ifdef __BORLANDC__
    #pragma hdrstop
#endif
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif
// Includes do trabalho
// Para manipulao de eventos temporais
#include <wx/timer.h>
// Para manipulao de imagens
#include <wx/image.h>
// Para mostrar a quantidade de ouro nas minas e nos depsitos
#include "statusFrame.h"
// Classe que define o ambiente (em relao ao problema)
#include "robosMineradores.h"
// Para toolbars (barra de ferramentas)
#include <wx/toolbar.h>
// Para o controle spin control
#include <wx/spinctrl.h>

// classe de aplicao
class robosApp;

// Define a janela principal do programa
class FrameRobosMineradores: public wxFrame
{
    public:
    FrameRobosMineradores(wxSize size = wxDefaultSize);
    ~FrameRobosMineradores();

    // Eventos dos botes da interface grfica
    
    // Mostra as quantidades atuais de ouro nas minas e depsitos
    void OnStatus( wxCommandEvent &event );
    // Mostra uma janela com informaes sobre o desenvolvimento do programa
    void OnSobre( wxCommandEvent &event );
    // Sai do programa
    void OnSair( wxCommandEvent &event );
    // Inicia a simulao
    void OnIniciar( wxCommandEvent &event );
    // Gera minas, depsitos e robs em posies aleatrias
    void OnGerar( wxCommandEvent &event );
    // Permite a insero de uma mina no ambiente
    void OnToolMina( wxCommandEvent &event );
    // Permite a insero de um depsito no ambiente
    void OnToolDeposito( wxCommandEvent &event );
    // Permite a insero de um rob no ambiente
    void OnToolRobo( wxCommandEvent &event );
    // Permite a insero de uma rvore (funciona como obstculo) no ambiente
    void OnToolArvore( wxCommandEvent &event );
    // Limpa todo o ambiente
    void OnToolApagar( wxCommandEvent &event );
    // Permite modificar as caractersticas do Tabu Search (como raio tabu, ...)
   void OnToolConf( wxCommandEvent &event );
    
    // Chamado quando o usurio clica no X para fechar
    void OnExit( wxCloseEvent& event );

    // Temporizador do sistema
    void OnTimer( wxTimerEvent &event );
    // Evento de clicar com o boto esquerdo do mouse. Para permitir a insero
    // de objetos no ambiente
    void OnMouseLeftUp( wxMouseEvent& event );
    // Funo que desenha toda a janela
    void OnPaint( wxPaintEvent& event );

private:

    // Funo que desenha elementos (mina, depsito, rob ou rvores) no ambiente
    void DesenharElementos(int tipo, wxDC* dc = NULL);

    // Objeto da classe robosMineradores que realmente faz o processamento
    clRobosMineradores robosMineradores;

    // Referncias aos elementos da GUI
    // imagens
    wxImage* imagemMina;
    wxImage* imagemRoboCheio;
    wxImage* imagemRoboVazio;
    wxImage* imagemDeposito;
    wxImage* imagemArvore;
    // barra de ferramentas
    wxToolBar *tb;

    // guarda a posio da ltima rvore desenhada
    int posXArvore, posYArvore;

    // altura e largura da real rea de simulao
    int largura, altura;
    // clock do temporizador
    int CLOCK;

    // objeto temporizador
    wxTimer timer;
    // janela de status
    statusFrame *status;
    // imagem formada pelos objetos que tem posio fixa (todos exceto robs)
    wxImage* background;

    // macros necessrias para trabalhar eventos
    DECLARE_DYNAMIC_CLASS(FrameRobosMineradores)
    DECLARE_EVENT_TABLE()
};


// classe de aplicao
class robosApp : public wxApp
{
    // inicia a aplicacao
    virtual bool OnInit();
    // ponteiro para a janela principal
    FrameRobosMineradores *framePrincipal;
};

// programa principal
IMPLEMENT_APP(robosApp)

// Identificadores 
enum { ID_TIMER = 10001, ID_SOBRE, ID_STATUS, ID_ROBO, ID_MINA, ID_DEPO, ID_CONF,
   ID_INICIAR, ID_PAUSAR, ID_GERAR, ID_APAGAR, ID_ARV, ID_AJUDA, };

// implementa a janela
IMPLEMENT_DYNAMIC_CLASS(FrameRobosMineradores, wxFrame)

// tabela de eventos
BEGIN_EVENT_TABLE(FrameRobosMineradores, wxFrame)
   
    // enventos da barra de ferramentas
    
    EVT_TOOL(ID_INICIAR,FrameRobosMineradores::OnIniciar)
    EVT_TOOL(ID_SOBRE,  FrameRobosMineradores::OnSobre)
    EVT_TOOL(ID_STATUS, FrameRobosMineradores::OnStatus)
    EVT_TOOL(wxID_EXIT, FrameRobosMineradores::OnSair)

    EVT_TOOL(ID_DEPO, FrameRobosMineradores::OnToolDeposito)
    EVT_TOOL(ID_MINA, FrameRobosMineradores::OnToolMina)
    EVT_TOOL(ID_ROBO, FrameRobosMineradores::OnToolRobo)
    EVT_TOOL(ID_ARV, FrameRobosMineradores::OnToolArvore)
    EVT_TOOL(ID_APAGAR, FrameRobosMineradores::OnToolApagar)
    EVT_TOOL(ID_CONF, FrameRobosMineradores::OnToolConf)
    EVT_TOOL(ID_INICIAR, FrameRobosMineradores::OnIniciar)
    EVT_TOOL(ID_GERAR, FrameRobosMineradores::OnGerar)

    // evento temporizador
    EVT_TIMER(ID_TIMER, FrameRobosMineradores::OnTimer)
    // evento do boto esquerdo do mouse
    EVT_LEFT_UP(FrameRobosMineradores::OnMouseLeftUp)
    // evento de fechar a janela
    EVT_CLOSE(FrameRobosMineradores::OnExit)
    // evento para desenhar a janela
    EVT_PAINT(FrameRobosMineradores::OnPaint)
END_EVENT_TABLE()


// Janela principal do programa
FrameRobosMineradores::FrameRobosMineradores(wxSize size)
        : wxFrame((wxFrame *)NULL, -1, "IA - Robs Mineradores", wxPoint(0,0), size),
          timer(this, ID_TIMER)
{
   Maximize(true); // maximiza a janela (Somente em Windows).
    
    // cria um objeto que conter todas as imagens utilizadas pelo programa.
    wxImage figuras(31*4,31*4);

#if wxUSE_GIF
    // carrega o arquivo de imagens
    if ( !figuras.LoadFile(wxString("figuras.gif")) )
        wxLogError("No foi possvel carregar \"figuras.gif\"");
#endif
    // Cada imagem  carregada a partir da figura principal
    imagemMina = new wxImage(figuras.GetSubImage(wxRect(wxPoint(62,31), wxPoint(93,62))));
    imagemDeposito = new wxImage(figuras.GetSubImage(wxRect(wxPoint(93,31), wxPoint(124,62))));
    imagemRoboCheio = new wxImage(figuras.GetSubImage(wxRect(wxPoint(0,31), wxPoint(31,62))));
    imagemRoboVazio = new wxImage(figuras.GetSubImage(wxRect(wxPoint(31,31), wxPoint(62,62))));
    imagemArvore = new  wxImage(figuras.GetSubImage(wxRect(wxPoint(31,62), wxPoint(62,93))));
    
    // Criando e cionfigurando a barra de ferramentas
    tb = CreateToolBar( wxTB_HORIZONTAL|wxNO_BORDER | wxTB_FLAT | wxTB_3DBUTTONS);
    tb->SetToolBitmapSize(wxSize(31, 31));
    tb->SetMargins( 2, 2 );
    tb->AddTool( ID_INICIAR, figuras.GetSubImage(wxRect(wxPoint(0,0), wxPoint(31,31))).ConvertToBitmap(),
                 figuras.GetSubImage(wxRect(wxPoint(31,0), wxPoint(62,31))).ConvertToBitmap(),
                 true, -1, -1, NULL, "Iniciar", "Inicia a Simulao");
    tb->AddTool( ID_GERAR, figuras.GetSubImage(wxRect(wxPoint(93,62), wxPoint(123,93))).ConvertToBitmap(),
                 wxNullBitmap,false, -1, -1, NULL,"Gerar", "Gera elementos em posies aleatreos" );
    tb->AddTool(ID_APAGAR, figuras.GetSubImage(wxRect(wxPoint(0,93), wxPoint(31,123))).ConvertToBitmap(),
                wxNullBitmap,false, -1, -1, NULL,"Limpar", "Apaga todos os elementos do ambiente");
    tb->AddTool(ID_CONF, figuras.GetSubImage(wxRect(wxPoint(62,62), wxPoint(93, 93))).ConvertToBitmap(),
                wxNullBitmap,false, -1, -1, NULL,"Coniguraes", "Configura as especificaes do algoritmo Tabu Search");
    tb->AddSeparator();
    tb->AddTool( ID_MINA, imagemMina->ConvertToBitmap(), wxNullBitmap, true, -1, -1, NULL,
                 "Mina", "Permite a insero de minas no ambiente" );
    tb->AddTool( ID_ROBO, imagemRoboVazio->ConvertToBitmap(), wxNullBitmap, true, -1, -1, NULL,
                 "Robo", "Permite a insero de robos no ambiente" );
    tb->AddTool( ID_DEPO, imagemDeposito->ConvertToBitmap(), wxNullBitmap, true, -1, -1, NULL,
                 "Depsito", "Permite a insero de depsitos no ambiente" );
    tb->AddTool( ID_ARV, imagemArvore->ConvertToBitmap(), wxNullBitmap, true, -1, -1, NULL,
                 "rvore", "Permite a insero de rvores no ambiente" );
    tb->AddSeparator();
    tb->AddTool(ID_SOBRE, figuras.GetSubImage(wxRect(wxPoint(62,0), wxPoint(93, 31))).ConvertToBitmap(),
                wxNullBitmap,false, -1, -1, NULL,"Sobre", "Sobre o programa Robos Mineradores");
    tb->AddTool(ID_AJUDA, figuras.GetSubImage(wxRect(wxPoint(93,0), wxPoint(123, 31))).ConvertToBitmap(),
                wxNullBitmap,false, -1, -1, NULL,"Ajuda", "Exibe a ajuda do programa");
    tb->AddTool(wxID_EXIT, figuras.GetSubImage(wxRect(wxPoint(0,62), wxPoint(31, 93))).ConvertToBitmap(),
                wxNullBitmap,false, -1, -1, NULL,"Sair", "Sai do programa");
    tb->Realize();
    
    // cria a janela de status
    status = new statusFrame;
    
   GetClientSize(&largura, &altura);
    
    // cria o ambiente com o tamanho mximo permitido pela janela principal 
    robosMineradores.Criar(largura, altura);
    
    CLOCK = 50;
    
    // Preenchendo o fundo de verde
    background = new wxImage(largura, altura);
    for(int i=0; i < largura; i++)
    {
        for(int j=0; j < altura; j++)
            background->SetRGB(i, j, 88, 137, 88);
    }

    // cria barra de status
    CreateStatusBar(2);
    SetStatusText("Desenvolvido por Jlio Csar Alves e Anderson Rocha");
}

// Destroi a janela, liberando a memoria alocada
FrameRobosMineradores::~FrameRobosMineradores()
{
    delete status;
    delete background;
}

// Sai do programa
void FrameRobosMineradores::OnSair( wxCommandEvent &event )
{
    Destroy();
}

// Chamado quando o usurio clica no X para fechar
void FrameRobosMineradores::OnExit(wxCloseEvent& event)
{
    Destroy();
}
    
// Mostra as quantidades atuais de ouro nas minas e depsitos
void FrameRobosMineradores::OnStatus(wxCommandEvent &event)
{
    status->Show(!status->IsShown());
}

// Mostra uma janela com informaes sobre o desenvolvimento do programa
void FrameRobosMineradores::OnSobre( wxCommandEvent &event )
{
    wxMessageBox("Programa desenvolvido por:\n\tAnderson de Rezende Rocha\n\tJlio Csar Alves\n"
                 "\n\t<undersun, jcalves>@comp.ufla.br\n\n"
                 "Desenvolvido para a disciplina\n de Inteligncia Artificial\n"
                 "Curso de Cincia da Computao\nUniveridade Federal deLavras\n", "Robos Mineradores - Sobre");
}

// Inicia ou pausa a simulao
void FrameRobosMineradores::OnIniciar( wxCommandEvent &event )
{
    if(timer.IsRunning())
    {
        tb->ToggleTool(ID_INICIAR, false);
        timer.Stop();
        SetStatusText("Simulao parada", 1);
    }
    else
    {
        tb->ToggleTool(ID_INICIAR, true);
        SetStatusText("Simulao em execuo!", 1);
        timer.Start(CLOCK);
    }
}

// Gera minas, depsitos e robs em posies aleatrias
void FrameRobosMineradores::OnGerar( wxCommandEvent &event )
{
    bool estavaExecutando = timer.IsRunning();
    // para o timer
    timer.Stop();

    // caixa de dilogo que pergunta sobre o nmero de objetos a serem criados
    wxDialog dialogo(this, -1, "Configurando o Ambiente", wxDefaultPosition, wxDefaultSize);

    wxBoxSizer *item0 = new wxBoxSizer( wxVERTICAL );
    wxGridSizer *item1 = new wxGridSizer( 2, 0, 0 );

    wxStaticText *item2 = new wxStaticText(&dialogo, -1, "Nmero de Robs", wxDefaultPosition, wxDefaultSize, 0 );
    item1->Add( item2, 0, wxALIGN_CENTRE|wxALL, 5 );

    wxSpinCtrl *item3 = new wxSpinCtrl(&dialogo, -1, "", wxDefaultPosition, wxSize(100,-1), 0, 0, 100, 0 );
    item1->Add( item3, 0, wxALIGN_CENTRE|wxALL, 5 );

    wxStaticText *item4 = new wxStaticText(&dialogo, -1, "Nmero de Minas", wxDefaultPosition, wxDefaultSize, 0 );
    item1->Add( item4, 0, wxALIGN_CENTRE|wxALL, 5 );

    wxSpinCtrl *item5 = new wxSpinCtrl(&dialogo, -1, "", wxDefaultPosition, wxSize(100,-1), 0, 0, 100, 0 );
    item1->Add( item5, 0, wxALIGN_CENTRE|wxALL, 5 );

    wxStaticText *item6 = new wxStaticText(&dialogo, -1,"Nmero de Depsitos", wxDefaultPosition, wxDefaultSize,0);
    item1->Add( item6, 0, wxALIGN_CENTRE|wxALL, 5 );

    wxSpinCtrl *item7 = new wxSpinCtrl(&dialogo, -1, "", wxDefaultPosition, wxSize(100,-1), 0, 0, 100, 0 );
    item1->Add( item7, 0, wxALIGN_CENTRE|wxALL, 5 );

    item1->Add(5, 5, 0, wxALIGN_CENTRE|wxALL, 1); //espacadores
    item1->Add(5, 5, 0, wxALIGN_CENTRE|wxALL, 1);

    wxButton *botaoOK = new wxButton(&dialogo, wxID_OK, "OK");
    botaoOK->SetDefault();
    item1->Add( botaoOK, 0, wxALIGN_CENTRE|wxALL, 5 );

    wxButton *botaoCancelar = new wxButton(&dialogo, wxID_CANCEL, "Cancelar");
    item1->Add( botaoCancelar, 0, wxALIGN_CENTRE|wxALL, 5 );

    item0->Add( item1, 0, wxALL, 5 );

    dialogo.SetAutoLayout( TRUE );
    dialogo.SetSizer( item0 );
    item0->Fit( &dialogo );
    item0->SetSizeHints( &dialogo );

    dialogo.ShowModal();

    if(dialogo.GetReturnCode() == wxID_OK)
    {
        // insere os elementos no ambiente e desenha-os na tela
        
        robosMineradores.AdicionarRobos(item3->GetValue());
        robosMineradores.AdicionarMinas(item5->GetValue());
        robosMineradores.AdicionarDepositos(item7->GetValue());

        wxClientDC dc( this );
        PrepareDC( dc );

        DesenharElementos(ID_ROBO, &dc);
        DesenharElementos(ID_MINA);
        DesenharElementos(ID_MINA, &dc);
        DesenharElementos(ID_DEPO);
        DesenharElementos(ID_DEPO, &dc);
        SetStatusText("Elementos gerados", 1);
    }

    if(estavaExecutando)
    {
        // reinicia o timer
        timer.Start();
        SetStatusText("Simulao em execuo", 1);
    }
}

// Permite a insero de um depsito no ambiente
void FrameRobosMineradores::OnToolDeposito( wxCommandEvent &event )
{
    if(tb->GetToolState(ID_DEPO))
    {
        tb->ToggleTool(ID_MINA, false);
        tb->ToggleTool(ID_ROBO, false);
        tb->ToggleTool( ID_ARV, false);
        tb->ToggleTool(ID_DEPO, true);
        SetStatusText("Clique no ambiente para inserir um depsito", 1);
    }
    else
        tb->ToggleTool(ID_DEPO, false);
}

// Permite a insero de uma mina no ambiente
void FrameRobosMineradores::OnToolMina( wxCommandEvent &event )
{
    if(tb->GetToolState(ID_MINA))
    {
        tb->ToggleTool(ID_MINA, true);
        tb->ToggleTool(ID_ROBO, false);
        tb->ToggleTool(ID_DEPO, false);
        tb->ToggleTool( ID_ARV, false);
        SetStatusText("Clique no ambiente para inserir uma mina", 1);
    }
    else
        tb->ToggleTool(ID_MINA, false);
}

// Permite a insero de um rob no ambiente
void FrameRobosMineradores::OnToolRobo( wxCommandEvent &event )
{
    if(tb->GetToolState(ID_ROBO))
    {
        tb->ToggleTool(ID_MINA, false);
        tb->ToggleTool(ID_ROBO, true);
        tb->ToggleTool(ID_DEPO, false);
        tb->ToggleTool( ID_ARV, false);
        SetStatusText("Clique no ambiente para inserir um robo", 1);
    }
    else
        tb->ToggleTool(ID_ROBO, false);
}

// Permite a insero de uma rvore (funciona como obstculo) no ambiente
void FrameRobosMineradores::OnToolArvore( wxCommandEvent &event )
{
    if(tb->GetToolState(ID_ARV))
    {
        tb->ToggleTool(ID_MINA, false);
        tb->ToggleTool(ID_ROBO, false);
        tb->ToggleTool(ID_DEPO, false);
        tb->ToggleTool( ID_ARV, true);
        SetStatusText("Clique no ambiente para inserir uma rvore", 1);
    }
    else
        tb->ToggleTool(ID_ARV, false);
}

// Limpa todo o ambiente
void FrameRobosMineradores::OnToolApagar( wxCommandEvent &event )
{
    for(int i=0; i < largura; i++)
    {
        for(int j=0; j < altura; j++)
            background->SetRGB(i, j, 88, 137, 88); // fundo verde
    }
    robosMineradores.ReiniciarAmbiente();
    Refresh();
    SetStatusText("Ambiente reiniciado", 1);
}

// Permite modificar as caractersticas do Tabu Search (como raio tabu, ...)
void FrameRobosMineradores::OnToolConf( wxCommandEvent &event )
{
    bool estavaExecutando = timer.IsRunning();
    timer.Stop();
   
   wxString str;

    // caixa de dilogo para configurar o Tabu Search
    wxDialog dialogo(this, -1, "Configurando o Tabu Search", wxDefaultPosition, wxDefaultSize);

    wxBoxSizer *item0 = new wxBoxSizer( wxVERTICAL );
    wxGridSizer *item1 = new wxGridSizer( 2, 0, 0 );

    wxStaticText *item2 = new wxStaticText(&dialogo, -1, "Raio Tabu", wxDefaultPosition, wxDefaultSize, 0 );
    item1->Add( item2, 0, wxALIGN_CENTRE|wxALL, 5 );

   str = "";
   str << robosMineradores.GetRaioTabu();
    wxSpinCtrl *item3 = new wxSpinCtrl(&dialogo, -1, str, wxDefaultPosition, wxSize(100,-1), 0, 0, 100, 0 );
    item1->Add( item3, 0, wxALIGN_CENTRE|wxALL, 5 );

    wxStaticText *item4 = new wxStaticText(&dialogo, -1, "Nmero de pontos analisados", wxDefaultPosition, wxDefaultSize, 0 );
    item1->Add( item4, 0, wxALIGN_CENTRE|wxALL, 5 );

   str = "";
   str << robosMineradores.GetNroPontosAleatorios();
    wxSpinCtrl *item5 = new wxSpinCtrl(&dialogo, -1, str, wxDefaultPosition, wxSize(100,-1), 0, 0, 100, 0 );
    item1->Add( item5, 0, wxALIGN_CENTRE|wxALL, 5 );

    wxStaticText *item6 = new wxStaticText(&dialogo, -1,"Raio de Procura", wxDefaultPosition, wxDefaultSize,0);
    item1->Add( item6, 0, wxALIGN_CENTRE|wxALL, 5 );

   str = "";
   str << robosMineradores.GetRaioProcura();
    wxSpinCtrl *item7 = new wxSpinCtrl(&dialogo, -1, str, wxDefaultPosition, wxSize(100,-1), 0, 0, 100, 0 );
    item1->Add( item7, 0, wxALIGN_CENTRE|wxALL, 5 );

    item1->Add(5, 5, 0, wxALIGN_CENTRE|wxALL, 1); //espacadores
    item1->Add(5, 5, 0, wxALIGN_CENTRE|wxALL, 1);

    wxButton *botaoOK = new wxButton(&dialogo, wxID_OK, "OK");
    botaoOK->SetDefault();
    item1->Add( botaoOK, 0, wxALIGN_CENTRE|wxALL, 5 );

    wxButton *botaoCancelar = new wxButton(&dialogo, wxID_CANCEL, "Cancelar");
    item1->Add( botaoCancelar, 0, wxALIGN_CENTRE|wxALL, 5 );

    item0->Add( item1, 0, wxALL, 5 );

    dialogo.SetAutoLayout( TRUE );
    dialogo.SetSizer( item0 );
    item0->Fit( &dialogo );
    item0->SetSizeHints( &dialogo );

    dialogo.ShowModal();

    if(dialogo.GetReturnCode() == wxID_OK)
    {
        // Seta as configuraes do algoritmo na classe ambiente
        robosMineradores.SetConfTabu(item3->GetValue(), item5->GetValue(), item7->GetValue());
        SetStatusText("Configuraes atualizadas", 1);
    }

    if(estavaExecutando)
    {
        // reinicia o timer
        timer.Start();
        SetStatusText("Simulao em execuo", 1);
    }
}

// Funo que desenha toda a janela
void FrameRobosMineradores::OnPaint( wxPaintEvent& event )
{
    wxPaintDC dc( this );
    PrepareDC( dc );

    // desenha o fundo...
    dc.DrawBitmap(background->ConvertToBitmap(), 0, 0);
    // ... e os robs
    DesenharElementos(ID_ROBO, &dc);
}


// Temporizador do sistema
void FrameRobosMineradores::OnTimer(wxTimerEvent& event)
{
    wxClientDC dc( this );
    PrepareDC( dc );
    wxPen pen;
    
    // posicoes anteriores dos robos
    clPonto posAntigas[robosMineradores.GetNroRobos()];
    int x, y, x2, y2;

    for (int i = 0; i < robosMineradores.GetNroRobos(); i++)
        posAntigas[i].Set(robosMineradores.Robo(i)->GetPosicao().GetX(),
                          robosMineradores.Robo(i)->GetPosicao().GetY());

    // Faz cada rob executar dez passos
    if (!robosMineradores.ExecutarPassos(10))
    {
        tb->ToggleTool(ID_INICIAR, false);
        SetStatusText("Simulaao terminada", 1);
        timer.Stop();
    }
    
    // esta enorme funo desenha o fundo novamente nas regies que os robs acabaram
    // de desocupar
    for (int i = 0; i < robosMineradores.GetNroRobos(); i++)
    {
        x = posAntigas[i].GetX();
        y = posAntigas[i].GetY();
        x2 = robosMineradores.Robo(i)->GetPosicao().GetX();
        y2 = robosMineradores.Robo(i)->GetPosicao().GetY();
        if(x2 > x && y2 < y)
        {
            for(int i=x; i < x2; i++)
            {
                for(int j=y; j < y+31; j++)
                {
                    pen.SetColour(background->GetRed(i,j), background->GetGreen(i,j),background->GetBlue(i,j));
                    dc.SetPen(pen);
                    dc.DrawPoint(i,j);
                }
            }
            for(int i=x2; i < x+31; i++)
            {
                for(int j=y2+31; j < y+31; j++)
                {
                    pen.SetColour(background->GetRed(i,j), background->GetGreen(i,j),background->GetBlue(i,j));
                    dc.SetPen(pen);
                    dc.DrawPoint(i,j);
                }
            }
        }
        else if(x2 < x && y2 < y)
        {
            for(int i=x2+31; i < x+31; i++)
            {
                for(int j=y; j < y+31; j++)
                {
                    pen.SetColour(background->GetRed(i,j), background->GetGreen(i,j),background->GetBlue(i,j));
                    dc.SetPen(pen);
                    dc.DrawPoint(i,j);
                }
            }
            for(int i=x; i < x2+31; i++)
            {
                for(int j=y2+31; j < y+31; j++)
                {
                    pen.SetColour(background->GetRed(i,j), background->GetGreen(i,j),background->GetBlue(i,j));
                    dc.SetPen(pen);
                    dc.DrawPoint(i,j);
                }
            }
        }
        else if(x2 > x && y2 > y)
        {
            for(int i=x; i < x+31; i++)
            {
                for(int j=y; j < y2; j++)
                {
                    pen.SetColour(background->GetRed(i,j), background->GetGreen(i,j),background->GetBlue(i,j));
                    dc.SetPen(pen);
                    dc.DrawPoint(i,j);
                }
            }
            for(int i=x; i < x2; i++)
            {
                for(int j=y2; j < y+31; j++)
                {
                    pen.SetColour(background->GetRed(i,j), background->GetGreen(i,j),background->GetBlue(i,j));
                    dc.SetPen(pen);
                    dc.DrawPoint(i,j);
                }
            }
        }
        else if(x2 < x && y2 > y)
        {
            for(int i=x; i < x+31; i++)
            {
                for(int j=y; j < y2; j++)
                {
                    pen.SetColour(background->GetRed(i,j), background->GetGreen(i,j),background->GetBlue(i,j));
                    dc.SetPen(pen);
                    dc.DrawPoint(i,j);
                }
            }
            for(int i=x2+31; i < x+31; i++)
            {
                for(int j=y2; j < y+31; j++)
                {
                    pen.SetColour(background->GetRed(i,j), background->GetGreen(i,j),background->GetBlue(i,j));
                    dc.SetPen(pen);
                    dc.DrawPoint(i,j);
                }
            }
        }
        else if(y > y2)
        {
            for(int i=x; i < x+31; i++)
            {
                for(int j=y2+31; j < y+31; j++)
                {
                    pen.SetColour(background->GetRed(i,j), background->GetGreen(i,j),background->GetBlue(i,j));
                    dc.SetPen(pen);
                    dc.DrawPoint(i,j);
                }
            }
        }
        else if(y < y2)
        {
            for(int i=x; i < x+31; i++)
            {
                for(int j=y; j < y2; j++)
                {
                    pen.SetColour(background->GetRed(i,j), background->GetGreen(i,j),background->GetBlue(i,j));
                    dc.SetPen(pen);
                    dc.DrawPoint(i,j);
                }
            }
        }
        else if(x2 < x)
        {
            for(int i=x2+31; i < x+31; i++)
            {
                for(int j=y; j < y+31; j++)
                {
                    pen.SetColour(background->GetRed(i,j), background->GetGreen(i,j),background->GetBlue(i,j));
                    dc.SetPen(pen);
                    dc.DrawPoint(i,j);
                }
            }
        }
        else if(x2 > x)
        {
            for(int i=x; i < x2; i++)
            {
                for(int j=y; j < y+31; j++)
                {
                    pen.SetColour(background->GetRed(i,j), background->GetGreen(i,j),background->GetBlue(i,j));
                    dc.SetPen(pen);
                    dc.DrawPoint(i,j);
                }
            }
        }
    }

    // desenha os robs
    DesenharElementos(ID_ROBO, &dc);

    pen.SetColour( *(wxWHITE));
    dc.SetPen(pen);
    // desenhando as minas
    for(int i = 0; i < robosMineradores.GetNroMinas(); i++)
    {
        if (robosMineradores.Mina(i)->Vazia())
        {
            x = robosMineradores.Mina(i)->GetPosicao().GetX();
            y = robosMineradores.Mina(i)->GetPosicao().GetY();
            dc.DrawLine(x, y, x+31, y+31);
            dc.DrawLine(x+31, y, x, y+31);
        }
    }

    // desenhando os depositos
    for(int i = 0; i < robosMineradores.GetNroDepositos(); i++)
    {
        if(robosMineradores.Deposito(i)->Cheio())
        {
            x = robosMineradores.Deposito(i)->GetPosicao().GetX();
            y = robosMineradores.Deposito(i)->GetPosicao().GetY();
            dc.DrawLine(x, y, x+31, y+31);
            dc.DrawLine(x+31, y, x, y+31);
        }
    }

    // seta a janela de status se ela est ativa
    if (status->IsShown())
    {
        wxString str("Estado da simulao\n\n");

        for (int i = 0; i < robosMineradores.GetNroRobos(); i++)
        {
            str << "Robo " << robosMineradores.Robo(i)->GetId();
            if (robosMineradores.Robo(i)->Carregado())
                str << " carregado\n";
            else
                str << " vazio\n";
        }

        str << "\n";
        for (int i = 0; i < robosMineradores.GetNroMinas(); i++)
        {
            str << "Mina " << robosMineradores.Mina(i)->GetId();
            if (robosMineradores.Mina(i)->Vazia())
                str << " vazia\n";
            else
                str << " tem " << robosMineradores.Mina(i)->GetQuantidade() << " unidades de ouro\n";
        }

        str << "\n";
        for (int i = 0; i < robosMineradores.GetNroDepositos(); i++)
        {
            str << "Depsito " << robosMineradores.Deposito(i)->GetId();
            if (robosMineradores.Deposito(i)->Cheio())
                str << " cheio com " << robosMineradores.Deposito(i)->GetQuantidade() << " unidades de ouro\n";
            else
                str << " contm " << robosMineradores.Deposito(i)->GetQuantidade() << " unidades de ouro\n";
        }

        status->NovoStatus(str);
    }
}

// Evento de clicar com o boto esquerdo do mouse. Para permitir a insero
// de objetos no ambiente
void FrameRobosMineradores::OnMouseLeftUp( wxMouseEvent& event )
{
    wxClientDC dc( this );
    PrepareDC( dc );

    // desenha uma mina se a regio no est ocupada
    if( tb->GetToolState(ID_MINA) )
    {
        if( robosMineradores.AdicionarMina( clPonto(event.GetX(), event.GetY()), 10 ) )
        {
            DesenharElementos(ID_MINA);
            DesenharElementos(ID_MINA, &dc);
        }
        else
        {
            wxBell();
            SetStatusText("No  possvel colocar uma mina nesta posio", 1);
        }
    }
    // desenha um rob se a regio no est ocupada
    else if( tb->GetToolState(ID_ROBO) )
    {
        if( robosMineradores.AdicionarRobo( clPonto(event.GetX(), event.GetY()) ) )
        {
            DesenharElementos(ID_ROBO, &dc);
        }
        else
        {
            wxBell();
            SetStatusText("No  possvel colocar um rob nesta posio", 1);
        }
    }
    // desenha um depsito se a regio no est ocupada
    else if( tb->GetToolState(ID_DEPO) )
    {
        if( robosMineradores.AdicionarDeposito( clPonto(event.GetX(), event.GetY()), 20 ) )
        {
            DesenharElementos(ID_DEPO);
            DesenharElementos(ID_DEPO, &dc);
        }
        else
        {
            wxBell();
            SetStatusText("No  possvel colocar um depsito nesta posio", 1);
        }
    }
    // desenha uma rvores se a regio no est ocupada
    else if( tb->GetToolState(ID_ARV) )
    {
        if( robosMineradores.AtualizarRegiao(event.GetX(), event.GetY(), 31, 31, true) )
        {
            posXArvore = event.GetX();
            posYArvore = event.GetY();
            DesenharElementos(ID_ARV);
            DesenharElementos(ID_ARV, &dc);
        }
        else
        {
            wxBell();
            SetStatusText("No  possvel colocar uma rvore nesta posio", 1);
        }
    }
}

// tipo: 1 robo, 2 mina, 3 deposito. Se um dc  passado desenha na tela seno na imagem de fundo
void FrameRobosMineradores::DesenharElementos(int tipo, wxDC* dc)
{
    wxColour transparente(255, 0, 255);
    wxColour cor;
    wxPen pen;
    wxImage *imagem;
    int nroItens, x, y, i2, j2;

    // carrega a imagem de um rob
    if (tipo == ID_ROBO)
    {
        imagem = imagemRoboVazio;
        nroItens = robosMineradores.GetNroRobos();
    }
    // carrega a imagem de uma mina
    else if(tipo == ID_MINA)
    {
        nroItens = robosMineradores.GetNroMinas();
        imagem = imagemMina;
    }
    // carrega a imagem de um depsito
    else if(tipo == ID_DEPO)
    {
        imagem = imagemDeposito;
        nroItens = robosMineradores.GetNroDepositos();
    }
    // carrega a imagem de uma rvore
    else if(tipo == ID_ARV)
    {
        imagem = imagemArvore;
        nroItens = 1;
    }
    // isso no deve ocorrer
    else
    {
        wxLogError("Tipo de desenho no existe");
        return;
    }


    for(int i = 0; i < nroItens; i++)
    {
        // atualiza as posies
        if (tipo == ID_ROBO)
        {
            y = robosMineradores.Robo(i)->GetPosicao().GetY();
            x = robosMineradores.Robo(i)->GetPosicao().GetX();
            if(robosMineradores.Robo(i)->Carregado())
                imagem = imagemRoboCheio;
            else
                imagem = imagemRoboVazio;
        }
        else if(tipo == ID_MINA)
        {
            y = robosMineradores.Mina(i)->GetPosicao().GetY();
            x = robosMineradores.Mina(i)->GetPosicao().GetX();
        }
        else if(tipo == ID_DEPO)
        {
            y = robosMineradores.Deposito(i)->GetPosicao().GetY();
            x = robosMineradores.Deposito(i)->GetPosicao().GetX();
        }
        else if(tipo == ID_ARV)
        {
            y = posYArvore;
            x = posXArvore;
        }

        // desenha a imagem pixel por pixel (para facilitar o uso de transparncia
        i2 = 0;
        for(int i = x;  i < x+31; i++)
        {
            j2 = 0;
            for(int j = y; j < y+31; j++)
            {
                cor.Set(imagem->GetRed(i2, j2), imagem->GetGreen(i2, j2), imagem->GetBlue(i2, j2));
                if (cor != transparente)
                {
                    // desenha no fundo
                    if ( dc == NULL)
                        background->SetRGB(i, j, imagem->GetRed(i2, j2), imagem->GetGreen(i2, j2),
                                           imagem->GetBlue(i2, j2));
                    else // desenha na tela
                    {
                        pen.SetColour(cor);
                        dc->SetPen(pen);
                        dc->DrawPoint(i, j);
                    }
                }
                else if (dc != NULL)
                {
                    // quando  transparente desenha o fundo na tela
                    pen.SetColour(background->GetRed(i, j), background->GetGreen(i, j),background->GetBlue(i, j));
                    dc->SetPen(pen);
                    dc->DrawPoint(i, j);
                }
                j2++;
            }
            i2++;
        }

    }
}

// Inicializa a janela principal
bool robosApp::OnInit()
{
#if wxUSE_GIF
    wxImage::AddHandler( new wxGIFHandler );
#endif

    int largura = 640;
    int altura = 480;
    // pega o tamanho rela do vdeo
    wxDisplaySize(&largura, &altura);

    framePrincipal = new FrameRobosMineradores( wxSize(largura-5, altura-30));
    framePrincipal->Show(TRUE);
    framePrincipal->Maximize(TRUE);

  return TRUE;
}
