/*******************************************************
*        Trabalho de Inteligncia Artificial           *
*             Anderson de Rezende Rocha                *
*                 Jlio Csar Alves                    * 
*******************************************************/

#include <time.h>
#include <list>
#include "mina.h"
#include "deposito.h"
#include "robo.h"
#include "matrizBool.h"

// Utilizado para o sorteio,  o ponto de partida que deve ser melhorado
const double INFINITO = 999999;

class clRobosMineradores
{
    public:
        // Aloca espao para um robo
        clRobosMineradores();
        // Constri um objeto robo a partir de certas confs do ambiente
        clRobosMineradores(int largAmbiente, int altAmbiente, int nRobos=0, int nMinas=0, int nDepositos=0);
        // Criar o ambiente
        void Criar(int largAmbiente, int altAmbiente, int nRobos=0, int nMinas=0, int nDepositos=0);
        // Desaloca a memoria
        ~clRobosMineradores();
    
        // Permite executar novamente a simulao
        void ReiniciarAmbiente();
        // Configura a execuo do algoritmos tais como raio analisado, nmero de pontos
        // aleatrios lanados e o raio de procura
      void SetConfTabu(int raio_tabu, int nro_pontos_aleatorios, int raio_procura);

        // Executa um certo nmero de passos do algoritmo
        bool ExecutarPassos(int nroPassos = 1);

        // Verifica se uma regio do ambiente est livre
        bool VerificarRegiaoLivre(int x, int y, int largura, int altura);
        // Atualiza uma regio
        bool AtualizarRegiao(int x, int y, int largura, int altura, bool ocupar = true);

        // Adiciona mais robos ao ambiente
        void AdicionarRobos(int nRobos = 1);
        // Adiciona um robo
        bool AdicionarRobo(clPonto posicao);
        // Adiciona mais depositos
        void AdicionarDepositos(int nDepositos = 1);
        bool AdicionarDeposito(clPonto posicao, int capacidade);
        // Adiciona mais minas
        void AdicionarMinas(int nMinas = 1);
        bool AdicionarMina(clPonto posicao, int quantidade);

        // Consultas ao sistema
        int GetNroMinas() {return nroMinas;}
        int GetNroDepositos() {return nroDepositos;}
        int GetNroRobos() {return nroRobos;}
      int GetRaioTabu() {return raioTabu;}
      int GetNroPontosAleatorios() {return nroPontosAleatorios;}
      int GetRaioProcura() {return raioProcura;}

        // Referencia aos depositos, mina e robos do ambiente. 
        clDeposito* Deposito(int i);
        clMina* Mina(int i);
        clRobo<clRobosMineradores>* Robo(int i);

    private:
        // Matriz que representa se um pixel est ou no ocupado
        clMatrizBool matriz;
        // Altura e largura do ambiente
        int alturaAmbiente;
        int larguraAmbiente;
      // Raio tabu utilizado pelo algoritmo
        int raioTabu;
      int nroPontosAleatorios;
      int raioProcura;

        // Representao das quantidades no sistema
        int nroMinas;
        int nroDepositos;
        int nroRobos;

        list<clDeposito> depositos;
        list<clMina> minas;
        list<clRobo<clRobosMineradores> > robos;
};

// Construtor padro para alocar espao
clRobosMineradores::clRobosMineradores() {}

clRobosMineradores::clRobosMineradores(int largAmbiente, int altAmbiente, int nRobos,
                                       int nMinas, int nDepositos)
{
    // Rotina que efetivamente cria os robos
    Criar(largAmbiente, altAmbiente, nRobos, nMinas, nDepositos);
}

// Construtor do ambiente
void clRobosMineradores::Criar(int largAmbiente, int altAmbiente, int nRobos,
                                       int nMinas, int nDepositos)
{
    nroMinas = 0;
    nroDepositos = 0;
    nroRobos = 0;

    // Medidas do ambiente
    alturaAmbiente = altAmbiente;
    larguraAmbiente = largAmbiente;
   
   // Configuraes padro do ambiente
    raioTabu = 1;
   nroPontosAleatorios = 5;
   raioProcura = 5;

    // Adiciona os componentes do ambiente
    AdicionarRobos(nRobos);
    AdicionarMinas(nMinas);
    AdicionarDepositos(nDepositos);

    // Cria a matriz de bool
    matriz.Criar(larguraAmbiente, alturaAmbiente);
    for(int i=0; i < larguraAmbiente; i++)
    {
        for(int j=0; j < alturaAmbiente; j++)
            matriz.SetItem(i, j, (i==0) || (j==0) || (i==larguraAmbiente-1) || (j==alturaAmbiente-1));
    }
}

// Destrutor
clRobosMineradores::~clRobosMineradores() {}

// Permite reiniciar uma execuo
void clRobosMineradores::ReiniciarAmbiente()
{
    for(int i=0; i < larguraAmbiente; i++)
    {
        for(int j=0; j < alturaAmbiente; j++)
            matriz.SetItem(i, j, (i==0) || (j==0) || (i==larguraAmbiente-1) || (j==alturaAmbiente-1));
    }

    // Destruindo minas, robos e depositos existentes
    list<clRobo<clRobosMineradores> >::iterator itRobos;

    // Atualizando os robos de modo que possam comecar de novo
    itRobos = robos.begin();
    while(itRobos != robos.end())
    {
        robos.remove(*itRobos);
        itRobos = robos.begin();
    }
    nroRobos = 0;

    // Atualizando as minas de modo que possam comecar de novo
    list<clMina>::iterator itMinas;

    itMinas = minas.begin();
    while(itMinas != minas.end())
    {
        minas.remove(*itMinas);
        itMinas = minas.begin();
    }
    nroMinas = 0;

    // Atualizando os depositos de modo que possam comecar de novo
    list<clDeposito>::iterator itDep;
    itDep = depositos.begin();
    while(itDep != depositos.end())
    {
        depositos.remove(*itDep);
        itDep = depositos.begin();
    }
    nroDepositos = 0;
}

// Configurando as condies do algoritmo Tabu_search
void clRobosMineradores::SetConfTabu(int raio_tabu, int nro_pontos_aleatorios, int raio_procura)
{
   raioTabu = raio_tabu;
   nroPontosAleatorios = nro_pontos_aleatorios;
   raioProcura = raio_procura;
   
   list<clRobo<clRobosMineradores> >::iterator itRobos;

   for(itRobos = robos.begin(); itRobos != robos.end(); itRobos++)
      (*itRobos).SetConfTabu(raioTabu, nroPontosAleatorios, raioProcura);
}   

// Executa um certo numero de passos do algoritmo
bool clRobosMineradores::ExecutarPassos(int nroPassos)
{
    bool ninguemAndou = true;

    for (int j = 0; j < nroPassos; j++)
    {
        for (int i = 0; i < nroRobos; i++)
        {
            if(Robo(i)->Andar())
                ninguemAndou = false;
        }
    }
    return !ninguemAndou;
}

// Retorna uma referncia ao depsito com este identificador. Serve
// para que a interface grfica possa desenh-lo
clDeposito* clRobosMineradores::Deposito(int i)
{
    if(i < nroDepositos)
    {
        list<clDeposito>::iterator it;
        for(it = depositos.begin(); (*it).GetId() != i; it++);
        return &(*it);
    }
    else
    {
        return NULL;
    }
}

// Retorna uma referncia a mina com este identificador. Serve
// para que a interface grfica possa desenh-la
clMina* clRobosMineradores::Mina(int i)
{
    if(i < nroMinas)
    {
        list<clMina>::iterator it;
        for(it = minas.begin(); (*it).GetId() != i; it++);
        return &(*it);
    }
    else
    {
        return NULL;
    }
}

// Retorna uma referncia ao robo com este identificador. Serve
// para que a interface grfica possa desenh-lo
clRobo<clRobosMineradores>* clRobosMineradores::Robo(int i)
{
    if(i < nroRobos)
    {
        list<clRobo<clRobosMineradores> >::iterator it;
        for(it = robos.begin(); (*it).GetId() != i; it++);
        return &(*it);
    }
    else
    {
        return NULL;
    }
}

// Permite adicionar um certo nmero de robos ao ambiente em tempo de execuo
void clRobosMineradores::AdicionarRobos(int nRobos)
{
    if(nRobos>0)
    {
        srand(time(0));
        int x, y;

        for (int i = 0; i < nRobos; i++)
        {
            do
            {
                x = ( rand()%(larguraAmbiente-32) );
                y = ( rand()%(alturaAmbiente-50) );
            } while(!AdicionarRobo(clPonto(x, y)));
        }
    }
}

// Permite adicionar um robo ao ambiente em tempo de execuo
bool clRobosMineradores::AdicionarRobo(clPonto posicao)
{
    if(AtualizarRegiao(posicao.GetX(), posicao.GetY(), 31, 31, true))
    {
        clRobo<clRobosMineradores> robo(this, nroRobos, posicao);
      robo.SetConfTabu(raioTabu, nroPontosAleatorios, raioProcura);
        robos.insert(robos.end(), robo);
        nroRobos++;
        return true;
    }
    return false;
}

// Permite adicionar um deposito ao ambiente em tempo de execuo
bool clRobosMineradores::AdicionarDeposito(clPonto posicao, int capacidade)
{
    if(AtualizarRegiao(posicao.GetX(), posicao.GetY(), 31, 31, true))
    {
        clDeposito deposito(nroDepositos, posicao, capacidade);
        depositos.insert(depositos.end(), deposito);
        nroDepositos++;
        return true;
    }
    return false;
}

// Permite adicionar uma mina ao ambiente em tempo de execuo
bool clRobosMineradores::AdicionarMina(clPonto posicao, int quantidade)
{
    if(AtualizarRegiao(posicao.GetX(), posicao.GetY(), 31, 31, true))
    {
        clMina mina(nroMinas, posicao, quantidade);
        minas.insert(minas.end(), mina);
        nroMinas++;
        return true;
    }
    return false;
}

// Permite adicionar um certo nmero de minas ao ambiente em tempo de execuo
void clRobosMineradores::AdicionarMinas(int nMinas)
{
    if(nMinas > 0)
    {
        srand(time(0));
        clPonto ponto;
        int x, y;

        for (int i = 0; i < nMinas; i++)
        {
            do
            {
                x = ( rand()%(larguraAmbiente-32) );
                y = ( rand()%(alturaAmbiente-50) );
            } while(!AdicionarMina(clPonto(x, y), 1+(rand()%10)));
        }
    }
}

// Permite adicionar um certo nmero de depsitos ao ambiente em tempo de execuo
void clRobosMineradores::AdicionarDepositos(int nDepositos)
{
    if(nDepositos > 0)
    {
        srand(time(0));
        clPonto ponto;
        int x, y;

        for (int i = 0; i < nDepositos; i++)
        {
            do
            {
                x = ( rand()%(larguraAmbiente-32) );
                y = ( rand()%(alturaAmbiente-50) );
            } while(!AdicionarDeposito(clPonto(x, y), (10+rand()%20)));
        }
    }
}

// Verifica de uma regio do ambiente est livre baseado na matriz de ocupao
// representada pela matriz de bool
bool clRobosMineradores::VerificarRegiaoLivre(int x, int y, int largura, int altura)
{
    for(int i = x; i < x+largura; i++)
    {
        for(int j = y; j < y+altura; j++)
        {
            if(matriz.GetItem(i, j))
                return false;
        }
    }
    return true;
}


// Dado que uma nova regio na interface foi atualizada, a matriz de bool representando a ocupao
// tambm deve ser configurada para tal
bool clRobosMineradores::AtualizarRegiao(int x, int y, int largura, int altura, bool ocupar)
{
    if( (!VerificarRegiaoLivre(x, y, largura, altura)) && ocupar)
        return false;

    for(int i = x; i < x+largura; i++)
    {
        for(int j = y; j < y+altura; j++)
        {
            matriz.SetItem(i, j, ocupar);
        }
    }
    return true;
}

